local math2d = require("__core__.lualib.math2d")
local vector = math2d.position

--[[
  Uses `LuaEntity::stop_spider()` to stop as soon as the spidertron reaches its destination.
  Since the game counts a spidertron as having reached its destination when it is 2 tiles away,
  we must extend the autopilot_destination 2 tiles beyond the given destination.
]]

local function on_player_used_spider_remote(event)
  if not event.success then return end
  do return end
  if not settings.global["spidertron-enhancements-show-stop-at-destination"].value then return end

  local spidertron = event.vehicle

  local autopilot_destinations = spidertron.autopilot_destinations
  local num_destinations = #autopilot_destinations
  local start_position
  if num_destinations > 1 then
    start_position = autopilot_destinations[num_destinations - 1]
  else
    start_position = spidertron.position
  end
  local end_position = autopilot_destinations[num_destinations]

  local start_to_end = vector.subtract(end_position, start_position)
  local unit_vector = vector.get_normalised(start_to_end)
  local double_vector = vector.multiply_scalar(unit_vector, 2)
  local beyond_position = vector.add(start_to_end, double_vector)

  for i, autopilot_destination in pairs(autopilot_destinations) do
    if i == 1 then
      -- Clear current autopilot_destinations by setting it directly
      if i == num_destinations then
        spidertron.autopilot_destination = beyond_position
      else
        spidertron.autopilot_destination = autopilot_destination
      end
    else
      if i == num_destinations then
        spidertron.add_autopilot_destination(beyond_position)
      else
        spidertron.add_autopilot_destination(autopilot_destination)
      end
    end
  end
end

local function on_spider_command_completed(event)
  do return end
  if not settings.global["spidertron-enhancements-show-stop-at-destination"].value then return end
  local spidertron = event.vehicle
  spidertron.stop_spider()
end

return {on_player_used_spider_remote = on_player_used_spider_remote, on_spider_command_completed = on_spider_command_completed}